package org.xbib.interlibrary.api;

import java.io.Closeable;
import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.ServiceLoader;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ApplicationModules {

    private static final Logger logger = Logger.getLogger(ApplicationModules.class.getName());

    private final Map<String, Object> applicationModules;

    public ApplicationModules() {
        this.applicationModules = new LinkedHashMap<>();
        for (ApplicationModule applicationModule : ServiceLoader.load(ApplicationModule.class)) {
            add(applicationModule);
        }
    }

    public <M extends ApplicationModule> void add(M module) {
        applicationModules.put(module.getName(), module);
    }

    public <M extends ApplicationModule> void remove(M module) {
        applicationModules.remove(module.getName());
    }

    @SuppressWarnings("unchecked")
    public <M extends ApplicationModule> M get(String name) {
        return (M) applicationModules.get(name);
    }

    @SuppressWarnings("unchecked")
    public <M extends ApplicationModule> M get(Class<M> cl) {
        return (M) applicationModules.values().stream()
                .filter(cl::isInstance)
                .findFirst()
                .orElse(null);
    }

    public void close() {
        for (Object object : applicationModules.values()) {
            if (object instanceof Closeable closeable) {
                try {
                    closeable.close();
                } catch (IOException e) {
                    logger.log(Level.SEVERE, e.getMessage(), e);
                }
            }
        }
    }
}
