package org.xbib.interlibrary.api.action;

import java.io.IOException;
import java.util.List;
import java.util.concurrent.CompletableFuture;

/**
 * An action.
 *
 * @param <Req> the request
 * @param <Resp> the response
 */
public interface Action<Req extends Request, Resp extends Response<Req>, B extends ResponseBuilder<Req, Resp>, Serv extends Service<Req, Resp, B>> {

    Action<Req, Resp, B, Serv> getInstance();

    void init(ActionArguments arguments);

    List<Serv> getServices();

    B newResponseBuilder(Req request);

    B newEmptyResponseBuilder(Req request);

    B newErrorResponse(Req request, Throwable throwable);

    Req validate(Req request);

    List<Resp> all(Req request);

    Resp any(Req request);

    Resp reduce(Req request);

    Resp allWithRetry(Req request);

    List<CompletableFuture<B>> submit(Req request);

    void close() throws IOException;
}
