package org.xbib.interlibrary.api.action.worker;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

public class WorkerThreadPoolExecutor extends ThreadPoolExecutor {

    private static final Logger logger = Logger.getLogger(WorkerThreadPoolExecutor.class.getName());

    private int returnCode;

    public WorkerThreadPoolExecutor(int maxThreads,
                                    BlockingQueue<Runnable> blockingQueue,
                                    ThreadFactory threadFactory) {
        super(0, maxThreads, 0L, TimeUnit.SECONDS, blockingQueue, threadFactory);
        this.returnCode = 0;
    }

    @Override
    protected void afterExecute(Runnable runnable, Throwable terminationCause) {
        super.afterExecute(runnable, terminationCause);
        Throwable throwable = terminationCause;
        if (throwable == null && runnable instanceof Future<?>) {
            try {
                Future<?> future = (Future<?>) runnable;
                if (future.isDone()) {
                    future.get();
                }
            } catch (CancellationException ce) {
                logger.log(Level.FINEST, ce.getMessage(), ce);
                throwable = ce;
            } catch (ExecutionException ee) {
                logger.log(Level.FINEST, ee.getMessage(), ee);
                throwable = ee.getCause();
            } catch (InterruptedException ie) {
                Thread.currentThread().interrupt();
                logger.log(Level.FINEST, ie.getMessage(), ie);
            }
        }
        if (throwable != null) {
            logger.log(Level.SEVERE, throwable.getMessage(), throwable);
            this.returnCode = 1;
        }
    }

    public int getReturnCode() {
        return returnCode;
    }
}
