package org.xbib.interlibrary.api.config;

import java.util.EnumSet;
import java.util.List;
import org.xbib.interlibrary.api.action.Requester;
import org.xbib.interlibrary.api.library.AddressStyle;
import org.xbib.interlibrary.api.library.Library;
import org.xbib.interlibrary.api.library.LibraryGroups;
import org.xbib.interlibrary.api.library.LibraryGroupsProvider;
import org.xbib.interlibrary.api.library.LibraryLocationsProvider;
import org.xbib.interlibrary.api.library.LibraryProvider;
import org.xbib.interlibrary.api.library.Station;
import org.xbib.settings.Settings;

import java.io.IOException;
import java.util.Collection;
import java.util.Map;

public interface Configuration {

    void init(Collection<LibraryProvider> libraryProviders,
              Collection<LibraryGroupsProvider> libraryGroupsProviders,
              Collection<LibraryLocationsProvider> libraryLocationsProviders);

    Settings getSettings();

    Collection<LibraryGroups> getLibraryGroups();

    Collection<String> getAllLibraryIdentifiers();

    Collection<String> getLibraryIdentifiers(String group) throws IOException;

    Library getLibrary(String isil);

    List<String> getLibraryAddress(String isil, AddressStyle addressStyle);

    Map<String, Requester> getRequesterMap();

    Collection<String> getSelf(String isil);

    Collection<String> getLocal(String isil);

    Collection<String> getAccept(String isil);

    Collection<String> getPrefer(String isil);

    boolean isRegisteredInterlibrary(String isil, boolean defaultValue);

    boolean isCopyInterlibrary(String isil, boolean defaultValue);

    boolean isOnlineInterlibrary(String isil, boolean defaultValue);

    boolean isSubLibrary(String isil, boolean defaultValue);

    boolean isLastResortLibrary(String isil, boolean defaultValue);

    boolean isOrganization(String isil, boolean defaultValue);

    boolean isPublic(String isil, boolean defaultValue);

    Collection<String> getAccounts() throws IOException;

    Collection<String> getConfiguredAccounts() throws IOException;

    Collection<String> getLocations(String isil, EnumSet<Station.Process> processes) throws IOException;

    void addLocation(String isil, Station.Process process, String location) throws IOException;

    void removeLocation(String isil, Station.Process process, String location) throws IOException;

    Collection<Station> getStations(String isil) throws IOException;

    void addStation(Station station) throws IOException;

    void removeStation(Station station) throws IOException;

    Collection<String> getLibraryAddressesAsCsvLines() throws IOException;

}
