package org.xbib.interlibrary.api.library;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Pattern;

/**
 * The simple library implements the Library interface
 */
public class DefaultIndexLibrary implements IndexLibrary {

    private final Builder builder;

    private DefaultIndexLibrary(Builder builder) {
        this.builder = builder;
    }

    public static Builder builder() {
        return new Builder();
    }

    @Override
    public String getISIL() {
        return builder.isil;
    }

    @Override
    public Collection<String> getAliases() {
        return builder.aliases;
    }

    @Override
    public String getName() {
        return builder.name;
    }

    @Override
    public Domain getDomain() {
        return builder.domain;
    }

    public boolean isRegisteredInterlibrary() {
        return builder.isRegisteredInterlibrary;
    }

    public boolean isCopyInterlibrary() {
        return builder.isCopyInterlibrary;
    }

    public boolean isOnlineInterlibrary() {
        return builder.isOnlineInterlibrary;
    }

    @Override
    public Collection<String> getCollections() {
        return builder.collections;
    }

    @Override
    public Map<Pattern, Object> getFilter() {
        return builder.filter;
    }

    @Override
    public Map<String, Object> getStatusCodes() {
        return builder.statusCodes;
    }

    @Override
    public Map<String, Object> getSubLibraries() {
        return builder.subLibraries;
    }

    @Override
    public void addUnmappedStatusText(String statusText) {
        builder.unmappedStatusTexts.add(statusText);
    }

    @Override
    public Set<String> getUnmappedStatusTexts() {
        return builder.unmappedStatusTexts;
    }

    @Override
    public String toString() {
        return builder.isil;
    }

    public static class Builder {

        private String isil;

        private final Collection<String> aliases;

        private String name;

        private Domain domain;

        private boolean isRegisteredInterlibrary;

        private boolean isCopyInterlibrary;

        private boolean isOnlineInterlibrary;

        private Collection<String> collections;

        private Map<Pattern, Object> filter;

        private Map<String, Object> statusCodes;

        private Map<String, Object> subLibraries;

        private final Set<String> unmappedStatusTexts;

        Builder() {
            this.aliases = new ArrayList<>();
            this.unmappedStatusTexts = ConcurrentHashMap.newKeySet();
        }

        public Builder setISIL(String isil) {
            this.isil = isil;
            return this;
        }

        public Builder addAlias(String alias) {
            this.aliases.add(alias);
            return this;
        }

        public Builder setName(String name) {
            this.name = name;
            return this;
        }

        public Builder setDomain(Domain domain) {
            this.domain = domain;
            return this;
        }

        public Builder setRegisteredInterlibrary(boolean isRegisteredInterlibrary) {
            this.isRegisteredInterlibrary = isRegisteredInterlibrary;
            return this;
        }

        public Builder setCopyInterlibrary(boolean isCopyInterlibrary) {
            this.isCopyInterlibrary = isCopyInterlibrary;
            return this;
        }

        public Builder setOnlineInterlibrary(boolean isOnlineInterlibrary) {
            this.isOnlineInterlibrary = isOnlineInterlibrary;
            return this;
        }

        public Builder setCollections(Collection<String> collections) {
            this.collections = collections;
            return this;
        }

        public Builder setFilter(Map<Pattern, Object> filter) {
            this.filter = filter;
            return this;
        }

        public Builder setSubLibraries(Map<String, Object> subLibraries) {
            this.subLibraries = subLibraries;
            return this;
        }

        public Builder setStatusCodes(Map<String, Object> statusCodes) {
            this.statusCodes = statusCodes;
            return this;
        }

        public DefaultIndexLibrary build() {
            return new DefaultIndexLibrary(this);
        }
    }
}
