package org.xbib.interlibrary.api.library;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import org.xbib.datastructures.tiny.TinyMap;

public class DefaultLibraryPostalAddress
        extends DefaultLibraryNotificationAddress implements LibraryPostalAddress {

    private final List<String> containerInformation;

    private boolean isPublic;

    public DefaultLibraryPostalAddress() {
        this.containerInformation = new ArrayList<>();
    }

    public void addContainerInformation(String containerInformation) {
        this.containerInformation.add(containerInformation);
    }

    @Override
    public List<String> getContainerInformation() {
        return containerInformation;
    }

    public void setPublic(boolean isPublic) {
        this.isPublic = isPublic;
    }

    @Override
    public boolean isPublic() {
        return isPublic;
    }

    @Override
    public Map<String, Object> asMap() {
        TinyMap.Builder<String, Object> postal = TinyMap.builder();
        postal.putIfNotNull("name", getName());
        postal.putIfNotNull("shortname", getShortName());
        postal.putIfNotNull("street", getStreet());
        postal.putIfNotNull("zip", getZipCode());
        postal.putIfNotNull("place", getPlace());
        postal.putIfNotNull("postofficebox", getPostOfficeBox());
        postal.putIfNotNull("postofficeboxzip", getPostOfficeBoxZipCode());
        postal.putIfNotNull("country", getCountry());
        postal.putIfNotNull("province", getProvince());
        postal.putIfNotNull("public", isPublic());
        postal.putIfNotNull("email", getEmail());
        postal.putIfNotNull("emails", getEmails());
        postal.putIfNotNull("phone", getPhoneNumber());
        postal.putIfNotNull("container", getContainerInformation());
        postal.putIfNotNull("homepage", getHomePageURL());
        postal.putIfNotNull("openinghours", getOpeningHours());
        postal.putIfNotNull("openinghourscomments", getOpeningHoursComments());
        postal.putIfNotNull("latitude", getLatitude());
        postal.putIfNotNull("longitude", getLongitude());
        return postal.build();
    }

    @Override
    public List<String> asLines(AddressStyle addressStyle, List<String> prepend, List<String> append) {
        List<String> lines = super.asLines(addressStyle, prepend, List.of());
        if (getEmails() != null && !getEmails().isEmpty()) {
            for (Map.Entry<String, String> entry: getEmails().entrySet()) {
                if (entry.getKey() != null && !entry.getKey().isEmpty()) {
                    lines.add(entry.getKey() + ": " + entry.getValue());
                } else {
                    if (!getEmail().toLowerCase(Locale.ROOT).equals(entry.getValue())) {
                        lines.add(entry.getValue());
                    }
                }
            }
        }
        if (!getContainerInformation().isEmpty()) {
            lines.add(String.join(", ", getContainerInformation()));
        }
        lines.addAll(append);
        return lines;
    }
}
