package org.xbib.interlibrary.api.library;

import java.util.Objects;

public class DefaultPlace implements Place {

    private final String country;

    private final String province;

    private final String place;

    private final String zip;

    private final boolean isPublic;

    public DefaultPlace(String country,
                        String province,
                        String place,
                        String zip,
                        boolean isPublic) {
        this.country = Objects.requireNonNull(country, "no country for " + province + " " + place + " " + zip);
        this.province = Objects.requireNonNull(province, "no province for " + country + " " + place + " " + zip);
        this.place = Objects.requireNonNull(place, "no place for " + country + " " + place + " " + zip);
        this.zip = Objects.requireNonNull(zip, "no zip for " + country + " " + province + " " + place);
        this.isPublic = isPublic;
    }

    @Override
    public String getCountry() {
        return country;
    }

    @Override
    public String getProvince() {
        return province;
    }

    @Override
    public String getPlace() {
        return place;
    }

    @Override
    public String getZip() {
        return zip;
    }

    @Override
    public boolean isPublic() {
        return isPublic;
    }

    @Override
    public int hashCode() {
        return country.hashCode() ^ province.hashCode() ^ place.hashCode() ^ zip.hashCode() ;
    }

    @Override
    public boolean equals(Object object) {
        return object instanceof Place &&
                country.equals(((Place) object).getCountry()) &&
                province.equals(((Place) object).getProvince()) &&
                place.equals(((Place) object).getPlace()) &&
                matchZip(zip, ((Place) object).getZip());
    }

    @Override
    public String toString() {
        return place + ", " + province + ", " + country;
    }

    @Override
    public int compareTo(Place place) {
        return (country + "|" + province + "|" + this.place).compareTo(place.getCountry() + "|" + place.getProvince() + "|" + place.getPlace());
    }

    private static boolean matchZip(String zip1, String zip2) {
        return zip1 != null && zip2 != null && zip1.substring(0, 1).equals(zip2.substring(0, 1));
    }
}
