package org.xbib.interlibrary.api.library;

import org.xbib.interlibrary.api.util.Maps;

import java.util.Map;

public class IndexLibraryHelper {

    public IndexLibraryHelper() {
    }

    public void parseLibrary(DefaultIndexLibrary.Builder builder,
                             Map<String, Object> map) {
        String isil = Maps.getString(map, "008.H.e");
        if (isil == null) {
            return;
        }
        String organizationType = toOrganizationType(Maps.getString(map, "035.E.f"));
        if (organizationType == null) {
            return;
        }
        String state = toState(Maps.getString(map, "035.E.a"));
        if (state == null) {
            return;
        }
        String name = extractName(map);
        if (name == null) {
            return;
        }
        builder.setISIL(isil);
        builder.setName(name);
        if ("Adresse".equals(state)) {
            parseLibrary(builder, map, isil, organizationType);
        }
    }

    private static void parseLibrary(DefaultIndexLibrary.Builder builder,
                                     Map<String, Object> map,
                                     String isil,
                                     String organizationType) {
        String sigel = Maps.getString(map, "008.H.d");
        if (sigel != null) {
            builder.addAlias(sigel);
        }
        Maps.getListMap(map, "035.D").forEach(m -> {
            Object object = m.get("a");
            if (object != null) {
                builder.addAlias(object.toString());
            }
            object = m.get("b");
            if (object != null) {
                builder.addAlias(object.toString());
            }
        });
        String serviceName = null;
        String serviceMode = null;
        String serviceOrganization = null;
        String serviceDomain = null;
        String serviceOnline = null;
        // https://sigel.staatsbibliothek-berlin.de/vergabe/adressenformat/807
        // 035I $a Leihverkehrsregion
        // 035I $b Leihverkehrsart
        // 035I $c Zuständiges Verbundsystem
        // 035I $d Weitere(s) Verbundsystem(e)
        // 035I $e Fernleihindikator
        //   l = ja = uneingeschränkte Fernleihe, Kopie und Ausleihe
        //   a = ja, nur Ausleihe = Ausleihe von Bänden möglich, keine Kopien
        //   e = ja, nur Kopie, elektronischer Versand an Endnutzer möglich =
        //        Keine Ausleihe von Bänden, der Endnutzer erhält eine elektronische Kopie*
        //   k = ja, nur Kopie =
        //        Keine Ausleihe von Bänden, nur Papierkopien werden versandt
        //   n = nein = Keine Fernleihe
        // 035I $f Fernleihindikator Ausland
        // 035I $g Typ Online-Fernleihe
        //   R nur regional, verbundintern
        //   V verbundübergreifend (beinhaltet auch regional)
        for (Map<String, Object> m :  Maps.getListMap(map, "035.I")) {
            if (m.containsKey("a")) {
                serviceDomain = (String) m.get("a");
            }
            if (m.containsKey("b")) {
                serviceName = (String) m.get("b");
            }
            if (m.containsKey("c")) {
                serviceOrganization = (String) m.get("c");
            }
            if (m.containsKey("e")) {
                serviceMode = toServiceMode((String) m.get("e"));
            }
            if (m.containsKey("g")) {
                serviceOnline = toServiceOnline((String) m.get("g"));
            }
        }
        if ("Verbundsystem/-katalog".equals(organizationType) || "Nationalbibliothek".equals(organizationType)) {
            builder.setDomain(Domain.of(serviceDomain, serviceOrganization));
            builder.setRegisteredInterlibrary(true);
            builder.setCopyInterlibrary(true);
            builder.setOnlineInterlibrary(true);
        }
        boolean isRegisteredInterlibrary = "ÜLV".equals(serviceName);
        boolean isOnlineInterlibrary = serviceOnline == null || "verbundübergreifend".equals(serviceOnline);
        if (serviceMode != null && serviceMode.contains("keine Fernleihe")) {
            // this library explicitly refuses interlibrary services
            builder.setDomain(Domain.of(serviceDomain, serviceOrganization));
            builder.setRegisteredInterlibrary(false);
            builder.setCopyInterlibrary(false);
            builder.setOnlineInterlibrary(false);
        }
        // for interlibrary, we need an interlibrary service domain
        if (serviceDomain != null) {
            builder.setDomain(Domain.of(serviceDomain, serviceOrganization));
            builder.setRegisteredInterlibrary(isRegisteredInterlibrary);
            builder.setOnlineInterlibrary(isOnlineInterlibrary);
        } else {
            if (isRegisteredInterlibrary) {
                throw new IllegalArgumentException("library is registered for interlibrary, but has no domain: " + isil);
            }
        }
    }

    private static String toOrganizationType(String string) {
        if (string == null) {
            return null;
        }
        return switch (string) {
            case "11" -> "Nationalbibliothek";
            case "15" -> "Zentrale Fachbibliothek";
            case "21" -> "Regionalbibliothek";
            case "33" -> "Öffentliche Bibliothek";
            case "34" -> "Mediathek";
            case "36" -> "Öffentliche Bibliothek für besondere Benutzergruppen";
            case "39" -> "Fahrbibliothek";
            case "51" -> "Archiv (staatlich)";
            case "52" -> "Archiv (kommunal)";
            case "53" -> "Archiv (kirchlich)";
            case "54" -> "Archiv (Herrschafts-/Familienarchiv)";
            case "55" -> "Archiv (Wirtschaft)";
            case "56" -> "Archiv (Parlament, Partei, Stiftung oder Verband)";
            case "57" -> "Archiv (Medienarchiv)";
            case "58" -> "Archiv (Hochschule, Wissenschaft)";
            case "59" -> "Archiv (Sonstiges)";
            case "60" -> "Zentrale Universitätsbibliothek";
            case "65" -> "Abteilungsbibliothek, Institutsbibliothek, Fachbereichsbibliothek (Universität)";
            case "70" -> "Zentrale Hochschulbibliothek, nicht Universität";
            case "73" -> "Abteilungsbibliothek, Fachbereichsbibliothek (Hochschule, nicht Universität)";
            case "81" -> "Wissenschaftliche Spezialbibliothek";
            case "82" -> "Einrichtung der Denkmalpflege";
            case "84" -> "Forschungseinrichtung";
            case "85" -> "Archiv (nicht Archivbibliothek)";
            case "86" -> "Museum (nicht Museumsbibliothek)";
            case "87" -> "Verlag";
            case "88" -> "Sonstige Einrichtung";
            case "89" -> "Paket elektronischer Ressourcen";
            case "91" -> "Fachstelle für Bibliotheken";
            case "94" -> "Regionaler Zentralkatalog / Leihverkehrszentrale";
            case "95" -> "Virtuelle Bibliothek / Portal";
            case "96" -> "Verbundsystem/-katalog";
            case "98" -> "Serviceeinrichtung";
            default -> throw new IllegalStateException("unknown organization type code: " + string);
        };
    }

    private static String toState(String string) {
        if (string == null) {
            return null;
        }
        return switch (string) {
            case "H" -> "Adresse";
            case "N" -> "Standort";
            case "I" -> "Information";
            case "P" -> "Produkt";
            default -> throw new IllegalStateException("unknown state code: " + string);
        };
    }

    private static String toServiceMode(String string) {
        if (string == null) {
            return null;
        }
        return switch (string) {
            case "l" -> "Fernleihe";
            case "n" -> "keine Fernleihe";
            case "a" -> "Fernleihe, nur Ausleihe";
            case "k" -> "Fernleihe, nur Kopie";
            case "e" -> "Fernleihe, elektronischer Kopienversand an Endnutzer";
            default -> throw new IllegalStateException("unknown service mode code: " + string);
        };
    }

    private static String toServiceOnline(String string) {
        if (string == null) {
            return null;
        }
        return switch (string) {
            case "R" -> "regional";
            case "V" -> "verbundübergreifend";
            default -> throw new IllegalStateException("unknown service online code: " + string);
        };
    }

    /**
     * 029.@ short name entry "Weitere Namensformen der Institution" $4 = "c" Kurzname
     * 029.A full name entry "Körperschaft - Bevorzugter Name"
     *
     * @param map the MARC record
     * @return the library name, preferably the short name, if not available, fall back to full name
     */
    private static String extractName(Map<String, Object> map) {
        String candidate = null;
        boolean found = false;
        for (Map<String, Object> m : Maps.getListMap(map, "029.@")) {
            if (m.containsKey("a")) {
                candidate = (String) m.get("a");
            }
            if ("c".equals(m.get("4"))) {
                found = true;
            }
            if (found && candidate != null) {
                return candidate;
            }
        }
        candidate = Maps.getString(map, "029.@.a");
        if (candidate != null) {
            return candidate;
        }
        // full name as fall back
        for (Map<String, Object> m : Maps.getListMap(map, "029.A")) {
            if (m.containsKey("a")) {
                return (String) m.get("a");
            }
        }
        return null;
    }
}
