package org.xbib.interlibrary.api.library;

import java.io.Serializable;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

public interface LibraryService extends Comparable<LibraryService> {

    int INVALID_PRIORITY = 9;

    String getId();

    Integer getPriority();

    void setISIL(String isil);

    String getISIL();

    String getName();

    String getKey();

    Mode getMode();

    List<Material> getMaterials();

    List<Location> getLocations();

    boolean containsYear(Integer year);

    boolean isValid();

    CharSequence getSimpleKey();

    CharSequence getRandomizedKey();

    Map<String, Object> asMap();

    interface Builder {

        Builder setId(String id);

        Builder setISIL(String isil);

        Builder setName(String name);

        Builder setKey(String key);

        Builder setMode(Mode mode);

        Builder setMaterials(List<Material> materials);

        Builder addLocation(Location location);

        Builder setDomainPriority(Integer domainPriority);

        Builder setLibraryPriority(Integer libraryPriority);

        Builder setSublibraryPriority(Integer sublibraryPriority);

        Builder setServicePriority(Integer servicePriority);

        LibraryService build();
    }

    interface Location {

        String getLibrary();

        String getLibraryName();

        String getCollection();

        String getCallnumber();

        String getTextualHoldings();

        String getDescription();

        List<LabeledUrl> getLinks();
    }

    interface LabeledUrl {

        String getUrl();

        String getLabel();
    }

    @SuppressWarnings("serial")
    class RandomizedKeyComparator implements Comparator<LibraryService>, Serializable {

        public RandomizedKeyComparator() {
        }

        @Override
        public int compare(LibraryService s1, LibraryService s2) {
            return s1.getRandomizedKey().toString().compareTo(s2.getRandomizedKey().toString());
        }
    }

    @SuppressWarnings("serial")
    class SimpleKeyComparator implements Comparator<LibraryService>, Serializable {

        public SimpleKeyComparator() {
        }

        @Override
        public int compare(LibraryService s1, LibraryService s2) {
            return s1.getSimpleKey().toString().compareTo(s2.getSimpleKey().toString());
        }
    }

    @SuppressWarnings("serial")
    class PriorityComparator implements Comparator<LibraryService>, Serializable {

        public PriorityComparator() {
        }

        @Override
        public int compare(LibraryService s1, LibraryService s2) {
            return s1.getPriority().compareTo(s2.getPriority());
        }
    }
}
