package org.xbib.interlibrary.api.library;

import java.util.Locale;
import java.util.logging.Level;
import java.util.logging.Logger;

public enum Mode {
    UNSPECIFIED(""),
    LOAN("a"),
    COPY("b"),
    LOAN_COPY("c"),
    NONE("d"),
    ELECTRONIC_COPY("e"),
    UNDEFINED("");

    private static final Logger logger = Logger.getLogger(Mode.class.getName());

    private final String code;

    Mode(String code) {
        this.code = code;
    }

    public String getCode() {
        return code;
    }

    public boolean isReturnable() {
        return "a".equals(code) || "c".equals(code);
    }

    public boolean isNonReturnable() {
        return "b".equals(code) || "c".equals(code) || "e".equals(code);
    }

    public boolean isForbidden() {
        return "d".equals(code);
    }

    public String toValue() {
        return name().toLowerCase(Locale.ROOT);
    }

    public static Mode fromValue(String value) {
        try {
            return value != null ? Mode.valueOf(value.toUpperCase(Locale.ROOT)) : UNDEFINED;
        } catch (IllegalArgumentException e) {
            logger.log(Level.WARNING, "unknown mode: " + value);
            return UNDEFINED;
        }
    }
}
