package org.xbib.interlibrary.api.order;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.Iterator;
import java.util.Map;

public interface OrderEvent {

    interface Entry {

        Long getId();

        OrderEventType getType();

        OrderEventCode getCode();

        String getCodeTranslation();

        LocalDateTime getTime();

        Status getStatus();

        String getMessage();

        String getOrderNumber();

        String getTransactionId();

        String getLibrary();

        String getUserId();

        String getRef();

        String toJson() throws IOException;

        Map<String, Object> asMap();
    }

    enum OrderEventType {
        ACK,
        DELIVERY,
        DELIVERYCOMMENT,
        EXPECT,
        MAIL,
        NOTIFY,
        REQUESTFORM,
        SUPPLY,
        TRANSACTION,
        VIEW
    }

    enum OrderEventCode {
        ANALYZE_PDF,
        ANALYZE_PDF_COLOR,
        ANALYZE_PDF_PAGE_FORMAT,
        ANALYZE_PDF_PAGE_ORIENTATION,
        ANALYZE_PDF_IMAGE,
        ANALYZE_PDF_BARCODE,
        AUTHOR,
        BEGIN_CONVERSIONS,
        CONVERT,
        MEMBERSHIP_DD,
        DELIVERY_COMMENT,
        DELIVERY_RANGE,
        DELIVERY_TYPE,
        DOI,
        DOWNLOAD,
        END_CONVERSIONS_FAIL,
        END_CONVERSIONS_SUCCESS,
        EXPECT_FAIL,
        EXPECT_SUCCESS,
        FAIL,
        FAIL_DOWNLOAD,
        HASH,
        ILL_SHIPPED,
        INHOUSE,
        INTITLE,
        ISBN,
        ISSN,
        ISSUE,
        MAIL_FAIL,
        MAIL_SUCCESS,
        MAXCOST,
        NOTIFICATION,
        NOTIFY,
        PAGINATION,
        PICKUPCODE,
        PICKUPLOCATION,
        PUBLISHER,
        PUBLISHERPLACE,
        REQUESTERCOMMENT,
        REQUESTEREMAIL,
        REQUESTERID,
        SHIPPED,
        SOURCE,
        SOURCEID,
        SUCCESS,
        SUCCESS_ANALYZE_PDF,
        SUCCESS_CONVERT,
        SUCCESS_DOWNLOAD,
        SUCCESS_UPLOAD,
        SUCCESS_VIEW,
        SUPPLY_FAIL,
        SUPPLY_SUCCESS,
        TAN,
        TITLE,
        TRANSACTION_FAIL,
        TRANSACTION_SUCCESS,
        VIEW,
        VOLUME,
        YEAR
    }

    Iterator<? extends Entry> getEntries();

    Iterator<? extends Entry> getEntriesInReverseOrder();

    String toJson() throws IOException;

    Map<String, Object> asMap();
}
