package org.xbib.interlibrary.api.order;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.EnumSet;
import java.util.List;
import java.util.function.Consumer;

import org.xbib.interlibrary.api.library.Role;
import org.xbib.interlibrary.api.route.Route;
import org.xbib.interlibrary.api.order.summary.Summary;
import org.xbib.interlibrary.api.order.tracking.DisplayStyle;
import org.xbib.interlibrary.api.order.tracking.TrackedOrder;
import org.xbib.interlibrary.api.order.tracking.TrackedOrderList;

public interface OrderManager {

    void getAllOrderNumbers(Consumer<String> consumer);

    void getCopyOrders(Consumer<String> consumer);

    String resolveTransactionId(String transactionId);

    String resolveIsilAndExternalReference(String isil, String externalReference);

    Order getOrder(String key);

    List<Order> getOrders(OrderSearchParameters searchParameters);

    TrackedOrder track(List<String> requesterids,
                       String transactionid,
                       DisplayStyle displayStyle) throws IOException;

    TrackedOrder track(List<String> requesterids,
                       String clientid,
                       int clientidLength,
                       String transactionid,
                       DisplayStyle displayStyle) throws IOException;

    TrackedOrderList trackList(List<String> requesterids,
                               String clientid,
                               int clientidLength,
                               EnumSet<Circulation> circulationSet,
                               Status status,
                               int days,
                               int offset,
                               int size,
                               DisplayStyle displayStyle);

    Summary getOrderSummary(String isil,
                            EnumSet<Circulation> circulations,
                            Role role,
                            LocalDateTime from,
                            LocalDateTime to);

    Requester getRequester(String key);

    Supply getSupply(String key);

    BibliographicData getBibliographicData(String key);

    History getHistory(String key);

    String createOrder(Order order);

    void updateOrder(Order order, Route route, String signComment) throws IOException;

    void updateOrderStatus(Order order, Status status, String comment) throws IOException;
}
