package org.xbib.interlibrary.api.order;

public interface Status {

    enum Code {
        UNDEFINED,
        PREPARING,
        ROUTING,
        /* "New" */
        INCOMING,
        /* "Workout" */
        PROCESSING,
        PROCESSED,
        SHIP,
        SIGN,
        RETURN,
        CANCELLED,
        /* "Not available" */
        NOTSUPPLIED,
        ERROR,
        END,
        EXPIRE,
        QUIT,
        INFO,
        IDLE
    }

    enum Substatus {
        UNDEFINED,
        /* error state */
        ERROR,
        /* Shipped but no delivery yet */
        INPROCESS,
        /* Shipped with delivery ready */
        READY,
        /* Ship with delivery probed */
        PROBED,
        /* Ship with delivery notified */
        NOTIFIED,
        /* set info text */
        INFO,
        /* set comment text */
        COMMENT,
        /* "Workout with request form created" */
        REQUESTFORM,
        /* Workout but preparing the delivery */
        PREPARE,
        /* Workout with successful transfer to library, MyBib Akzeptiert */
        TRANSFERRED,
        /* MyBib nicht akzeptiert */
        NOTTRANSFERRED
    }

    Code getCode();

    String getStatusText();

    String getStatusTranslation();

    Substatus getSubstatus();

    String getSubstatusText();

    String getSubstatusTranslation();

    Status UNDEFINED = new DefaultStatus(Code.UNDEFINED);

    Status INCOMING = new DefaultStatus(Code.INCOMING);

    Status PROCESSING = new DefaultStatus(Code.PROCESSING);

    Status PROCESSING_REQUESTFORM = new DefaultStatus(Code.PROCESSING, Substatus.REQUESTFORM);

    Status PROCESSING_PREPARETRANSFER = new DefaultStatus(Code.PROCESSING, Substatus.PREPARE);

    Status SHIP = new DefaultStatus(Code.SHIP);

    Status SHIP_INPROCESS = new DefaultStatus(Code.SHIP, Substatus.INPROCESS);

    Status SHIP_READY = new DefaultStatus(Code.SHIP, Substatus.READY);

    Status SHIP_PROBED = new DefaultStatus(Code.SHIP, Substatus.PROBED);

    Status NOTSUPPLIED = new DefaultStatus(Code.NOTSUPPLIED);

    Status CANCEL = new DefaultStatus(Code.CANCELLED);
}
