package org.xbib.interlibrary.api.order.handler;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.nio.charset.Charset;
import java.util.LinkedHashMap;
import java.util.Map;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import org.xbib.interlibrary.api.order.message.Message;

import static javax.xml.stream.XMLStreamConstants.CHARACTERS;
import static javax.xml.stream.XMLStreamConstants.END_ELEMENT;
import static javax.xml.stream.XMLStreamConstants.START_ELEMENT;

public abstract class AbstractXMLReceiver {

    public AbstractXMLReceiver() {
    }

    public void fromXML(InputStream inputStream,
                        Charset charset,
                        EventHandler<? extends Message> handler) throws XMLStreamException, IOException {
        Reader reader = new InputStreamReader(inputStream, charset);
        XMLInputFactory inputFactory = XMLInputFactory.newDefaultFactory();
        XMLStreamReader xmlStreamReader = inputFactory.createXMLStreamReader(reader);
        String elementName = "";
        StringBuilder sb = new StringBuilder();
        Map<String, String> params = new LinkedHashMap<>();
        while (xmlStreamReader.hasNext()) {
            int code = xmlStreamReader.next();
            switch (code) {
                case START_ELEMENT -> elementName = xmlStreamReader.getLocalName();
                case CHARACTERS -> sb.append(xmlStreamReader.getText());
                case END_ELEMENT -> {
                    switch (elementName) {
                        case "message-type", "requester-id", "requester-email-address", "requester-note",
                                "responder-id", "responder-note", "ill-service-type", "responder-specific-service",
                                "transaction-inital-req-id-symbol", "transaction-group-qualifier",
                                "transaction-sub-transaction-qualifier",
                                "client-identifier", "client-name", "client-status", "client-group",
                                "del-email-address", "client-delivery-location", "client-delivery-code",
                                "maximum-cost", "delivery-service", "notification-service",
                                "name-of-person-or-institution", "del-postal-name-of-person-or-institution",
                                "street-and-number", "del-postal-street-and-number", "post-office-box",
                                "del-postal-post-office-box", "city", "del-postal-city", "country",
                                "del-postal-country", "code", "del-postal-code", "del-notes", "item-title",
                                "item-place-of-publication", "item-publisher", "item-issn", "item-iSSN",
                                "item-isbn", "item-iSBN", "item-verification-reference-source",
                                "item-system-no", "item-publication-date-of-component", "item-volume-issue",
                                "item-pagination", "item-title-of-article", "item-article-title",
                                "item-author-of-article", "item-article-author", "supplier-id",
                                "request-form-id", "request-form-name", "request-form-street",
                                "request-form-postalcode", "request-form-city" ->
                                params.put(elementName, sb.toString());
                    }
                }
            }
        }
        receive(params, handler);
    }

    public void receive(Map<String, String> params,
                        EventHandler<? extends Message> eventHandler) throws IOException {
        String messageType = params.get("message-type");
        String requesterId = params.get("requester-id");
        String requesterEmail = params.get("requester-email-address");
        String responderId = params.get("responder-note");
        String illServiceType = params.get("ill-service-type");
        String responderSpecificService = params.get("responder-specific-service");
        // requester library symbol
        String transactionInitialReqIdSymbol = params.get("transaction-initial-req-id-symbol");
        String transactionGroupQualifier = params.get("transaction-group-qualifier");
        String transactionQualifier = params.get("transaction-qualifier");
        String transactionSubTransactionQualifier = params.get("transaction-sub-transaction-qualifier");
        String clientId = params.get("client-identifier");
        String clientName = params.get("client-name");
        String clientStatus = params.get("client-group");
        String clientEmail = params.get("client-email");
        String clientDeliveryLocation = params.get("client-delivery-location");
        String clientDeliveryCode = params.get("client-delivery-code");
        String maximumCost = params.get("maximum-cost");
        String deliveryService = params.get("delivery-service");
        String notificationService = params.get("notification-service");
        String nameOfPersonOrInstitution = params.get("name-of-person-institution");
        if (nameOfPersonOrInstitution == null) {
            nameOfPersonOrInstitution = params.get("del-postal-name-of-person-or-institution");
        }
        String extendedPostalDeliveryAddress = params.get("extended-postal-delivery-address");
        if (extendedPostalDeliveryAddress == null) {
            extendedPostalDeliveryAddress = params.get("del-postal-extended-postal-delivery-address");
        }
        String postalStreetAndNumber = params.get("street-and-number");
        if (postalStreetAndNumber == null) {
            postalStreetAndNumber = params.get("del-postal-street-and-number");
        }
        String postOfficBox = params.get("post-office-box");
        if (postOfficBox == null) {
            postOfficBox = params.get("bpost-office-box");
            if (postOfficBox == null) {
                postOfficBox = params.get("del-postal-post-office-box");
            }
        }
        String city = params.get("city");
        if (city == null) {
            city = params.get("del-postal-city");
        }
        String country = params.get("country");
        if (country == null) {
            country = params.get("del-postal-country");
        }
        String code = params.get("code");
        if (code == null) {
            code =params.get("del-postal-code");
        }
        String delNotes = params.get("del-notes");
        String itemTitle = params.get("item-title");
        //String itemSeriesTitleNumber =
        String itemPlaceOfPublication = params.get("item-place-of-publication");
        String itemPublisher = params.get("item-publisher");
        String itemIdentifier = params.get("item-issn");
        if (itemIdentifier == null) {
            itemIdentifier = params.get("item-iSSN");
            if (itemIdentifier == null) {
                itemIdentifier = params.get("item-isbn");
                if (itemIdentifier == null) {
                    itemIdentifier = params.get("item-iSBN");
                }
            }
        }
        String itemVerificationReferenceSource = params.get("item-verification-reference-source");
        String itemSystemNo = params.get("item-system-no");
        String date = params.get("item-publication-date-of-component");
        String itemVolumeIssue = params.get("item-volume-issue");
        String itemPagination = params.get("item-pagination");
        String itemTitleOfArticle = params.get("item-title-of-article");
        if (itemTitleOfArticle == null) {
            itemTitleOfArticle = params.get("item-article-title");
        }
        String itemAuthorOfArticle = params.get("item-author-of-article");
        if (itemAuthorOfArticle == null) {
            itemAuthorOfArticle = params.get("item-article-author");
        }
        String supplierId = params.get("supplier-id");
        String requesterFormId = params.get("request-form-id");
        String requesterFormName = params.get("request-form-name");
        String requesterFormStreet = params.get("request-form-street");
        String requesterFormPostalCode = params.get("request-form-postalcode");
        String requesterFormCity = params.get("request-form-city");


    }
}
