package org.xbib.interlibrary.api.util;

import java.util.Collection;
import java.util.Objects;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class Lists {

    private Lists() {
    }

    public static Collection<?> join(Object... objects) {
        return Stream.of(objects)
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
    }

    public static Collection<Collection<?>> append(Collection<?>... collections) {
        return Stream.of(collections)
                .filter(Objects::nonNull)
                .filter(Predicate.not(Collection::isEmpty))
                .collect(Collectors.toList());
    }

    public static Collection<?> flatten(Collection<?>... collections) {
        return Stream.of(collections)
                .filter(Objects::nonNull)
                .filter(Predicate.not(Collection::isEmpty))
                .flatMap(Collection::stream)
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
    }

    public static String firstElementAsStringIfAny(Object object) {
        if (object == null) {
            return null;
        }
        if (!(object instanceof Collection)) {
            return object.toString();
        }
        Collection<?> collection = (Collection<?>) object;
        return collection.stream().map(Object::toString).findFirst().orElse(null);
    }

    public static String joinWithDelimiter(String delimiter, Object... objects) {
        if (objects == null) {
            return null;
        }
        if (objects.length == 0) {
            return null;
        }
        if (objects[0] instanceof Collection) {
            return joinWithDelimiter(delimiter, (Collection<?>) objects[0]);
        }
        return Stream.of(objects)
                .filter(Objects::nonNull)
                .map(Object::toString)
                .collect(Collectors.joining(delimiter));
    }

    public static String joinWithDelimiter(String delimiter, Collection<?> collection) {
        return Stream.of(collection)
                .filter(Objects::nonNull)
                .filter(Predicate.not(Collection::isEmpty))
                .flatMap(Collection::stream)
                .filter(Objects::nonNull)
                .map(Object::toString)
                .collect(Collectors.joining(delimiter));
    }
}
