package org.xbib.interlibrary.api.util;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@SuppressWarnings("unchecked")
public class Maps {

    private Maps() {
    }

    public static String getString(Map<?, ?> map, String key) {
        Object object = get(map, key);
        return switch (object) {
            case null -> null;
            case List<?> list -> list.getFirst().toString();
            case Map<?, ?> map1 -> null;
            default -> object.toString();
        };
    }

    @SuppressWarnings("rawtypes")
    public static List<String> getStringList(Map<?, ?> map, String key) {
        Object object = get(map, key);
        return switch (object) {
            case null -> List.of();
            case List list -> list;
            case Map<?, ?> map1 -> List.of();
            default -> List.of(object.toString());
        };
    }

    public static Integer getInteger(Map<?, ?> map, String key) {
        Object object = get(map, key);
        if (object == null) {
            return -1;
        }
        try {
            if (object instanceof List) {
                return Integer.parseInt(((List<?>) object).getFirst().toString());
            }
            if (object instanceof Integer) {
                return (Integer) object;
            }
            return Integer.parseInt(object.toString());
        } catch (NumberFormatException e) {
            return null;
        }
    }

    public static Long getLong(Map<?, ?> map, String key) {
        Object object = get(map, key);
        if (object == null) {
            return -1L;
        }
        try {
            if (object instanceof List) {
                return Long.parseLong(((List<?>) object).getFirst().toString());
            }
            if (object instanceof Long) {
                return (Long) object;
            }
            return Long.parseLong(object.toString());
        } catch (NumberFormatException e) {
            return null;
        }
    }

    public static boolean getBoolean(Map<?, ?> map, String key) {
        Object object = get(map, key);
        if (object == null) {
            return false;
        }
        try {
            if (object instanceof List<?> list) {
                return Boolean.parseBoolean(list.isEmpty() ? "false" : list.getFirst().toString());
            } else if (object instanceof Boolean) {
                return (Boolean) object;
            } else {
                return Boolean.parseBoolean(object.toString());
            }
        } catch (NumberFormatException e) {
            return false;
        }
    }

    public static Collection<String> getList(Map<?, ?> map, String key) {
        Object o = getAll(map, key);
        if (o instanceof Collection) {
            return (Collection<String>) o;
        } else if (o != null) {
            return List.of(o.toString());
        }
        return List.of();
    }

    public static Map<?, ?> getMap(Map<?, ?> map, String key) {
        Object object = get(map, key);
        if (object == null) {
            return Map.of();
        }
        if (object instanceof Map) {
            return ((Map<?, ?>) object);
        }
        return Map.of("", object);
    }

    public static List<Map<String, Object>> getListMap(Map<String, Object> map, String key) {
        Object o = get(map, key);
        if (o instanceof List) {
            return (List<Map<String, Object>>) o;
        } else if (o instanceof Map) {
            return List.of((Map<String, Object>) o);
        }
        return List.of();
    }

    public static Collection<Object> collect(Map<?, ?> map,
                                             final String key,
                                             final String... subKeys) {
        Object object = get(map, key);
        if (object instanceof List) {
            return ((List<?>) object).stream()
                    .flatMap(t -> subKey(t, subKeys))
                    .collect(Collectors.toList());
        } else if (object instanceof Map) {
            return subKey(object, subKeys)
                    .collect(Collectors.toList());
        } else {
            return List.of();
        }
    }

    @SuppressWarnings("unchecked")
    public static Collection<Map<?, ?>> collectList(Map<?, ?> map,
                                                    final String key,
                                                    final String... subKeys) {
        return collect(map, key, subKeys).stream()
                .map(o -> o instanceof Map ? (Map<?, ?>) o : flatList((List<Map<?, ?>>) o))
                .collect(Collectors.toList());
    }

    public static Map<?, ?> flatList(List<Map<?, ?>> listOfMaps) {
        return listOfMaps.stream()
                .flatMap(m -> m.entrySet().stream())
                .collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
    }

    @SuppressWarnings("rawtypes") // for PMD
    private static Stream<?> subKey(Object object, String... subKeys) {
        return (!(object instanceof Map)) ?  null : Stream.of(subKeys)
                .map(((Map) object)::get)
                .filter(Objects::nonNull);
    }

    public static Collection<Object> collect(Map<?, ?> map,
                                             final CharSequence joiner,
                                             final String key,
                                             final String... subKeys) {
        Object object = get(map, key);
        if (object instanceof List) {
            return ((List<?>) object).stream()
                    .map(t -> subKey(t, subKeys)
                            .map(Objects::toString)
                            .collect(Collectors.joining(joiner)))
                    .collect(Collectors.toList());
        } else if (object instanceof Map) {
            return List.of(subKey(object, subKeys)
                    .map(Objects::toString)
                    .collect(Collectors.joining(joiner)));
        } else {
            return List.of();
        }
    }

    private static <T> T get(Map<?, ?> map, String key) {
        return get(map, key.split("\\."));
    }

    private static <T> T get(Map<?, ?> map, String[] keys) {
        if (map == null) {
            return null;
        }
        String key = keys[0];
        Object o = map.get(key);
        if (o == null) {
            return null;
        }
        if (!(o instanceof List)) {
            o = Collections.singletonList(o);
        }
        List<?> list = (List<?>) o;
        if (keys.length == 1) {
            return (T) list;
        }
        List<T> result = new LinkedList<>();
        for (Object oo : list) {
            if (oo instanceof Map<?, ?> m) {
                if (keys.length == 2) {
                    if (m.containsKey(keys[1])) {
                        T t = (T) m.get(keys[1]);
                        if (t instanceof List) {
                            for (Object ooo : ((List<?>) t)) {
                                T tt = (T) ooo;
                                result.add(tt);
                            }
                        } else {
                            result.add(t);
                        }
                    }
                } else {
                    Object ooo = get(m, Arrays.copyOfRange(keys, 1, keys.length));
                    if (ooo != null) {
                        T t = (T) ooo;
                        result.add(t);
                    }
                }
            } else if (oo instanceof List) {
                for (Object ooo : ((List<?>) oo)) {
                    T t = (T) ooo;
                    result.add(t);
                }
            } else {
                T t = (T) oo;
                result.add(t);
            }
        }
        if (!result.isEmpty()) {
            if (result.size() == 1) {
                return result.getFirst();
            }
            return (T) result;
        }
        return null;
    }

    private static <T> T getAll(Map<?, ?> map, String key) {
        return getAll(map, key.split("\\."));
    }

    @SuppressWarnings("unchecked")
    private static <T> T getAll(Map<?, ?> map, String[] keys) {
        if (map == null) {
            return null;
        }
        String key = keys[0];
        Object o = map.get(key);
        if (o == null) {
            return null;
        }
        if (!(o instanceof List)) {
            o = Collections.singletonList(o);
        }
        List<?> list = (List<?>) o;
        if (keys.length == 1) {
            return (T) list;
        }
        for (Object oo : list) {
            if (oo instanceof Map<?, ?> m) {
                if (keys.length == 2) {
                    if (m.containsKey(keys[1])) {
                        return (T) m.get(keys[1]);
                    }
                } else {
                    Object ooo = get(m, Arrays.copyOfRange(keys, 1, keys.length));
                    if (ooo != null) {
                        return (T) ooo;
                    }
                }
            } else {
                return (T) oo;
            }
        }
        return null;
    }
}
