package org.xbib.interlibrary.api.util;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;
import org.xbib.interlibrary.api.ApplicationModule;

public class TopologicalSort<M extends ApplicationModule> {

    private final List<M> modules;

    private final int vertices;

    private final LinkedList<Integer>[] adjList;

    @SuppressWarnings("unchecked")
    public TopologicalSort(List<M> modules) {
        this.modules = modules;
        this.vertices = modules.size();
        this.adjList = new LinkedList[vertices];
        for (int i = 0; i < vertices; i++) {
            adjList[i] = new LinkedList<>();
        }
        fill();
    }

    public List<M> sort() {
        boolean[] visited = new boolean[vertices];
        Stack<Integer> stack = new Stack<>();
        for (int i = 0; i < vertices; i++) {
            if (!visited[i]) {
                sort(i, visited, stack);
            }
        }
        List<M> result = new ArrayList<>();
        int size = stack.size();
        for (int i = 0; i < size; i++) {
            result.add(modules.get(stack.pop()));
        }
        return result;
    }

    private void sort(int start, boolean[] visited, Stack<Integer> stack) {
        visited[start] = true;
        for (int i = 0; i < adjList[start].size(); i++) {
            int vertex = adjList[start].get(i);
            if (!visited[vertex]) {
                sort(vertex, visited, stack);
            }
        }
        stack.push(start);
    }

    private void fill() {
        for (int i = 0; i < vertices; i++) {
            ApplicationModule module = modules.get(i);
            int n = modules.indexOf(module);
            addEdge(n, n);
            if (module.dependsOn() != null) {
                for (Class<? extends ApplicationModule> depClass : module.dependsOn()) {
                    modules.stream()
                            .filter(depClass::isInstance).findFirst()
                            .ifPresentOrElse(depModule ->
                                    addEdge(n, modules.indexOf(depModule)),
                                    () -> { throw new IllegalStateException("module not found: " + depClass);
                            });
                }
            }
        }
    }

    private void addEdge(int source, int destination) {
        if (destination < 0) {
            throw new IllegalArgumentException();
        }
        adjList[source].addFirst(destination);
    }
}
